/*

 SWR meter
 
 Juha Niinikoski OH2NLT 14.01.2004 code for HF amp proto

 Note if you use 16F877A chip add -NO_STRING_PACK option for correct putst() operation.

 ! Beware faulty 16F877A (shit) parts. Two weeks work for nothing to find this. ! 22.01.2004
                               Read Microchip Errata papers.

 You can use either 16F877A or 16F877 chips. With 16F877A you get better back light control

 Bar graph & peak display tests 22.01.2004
 SWR Meter code first release 26.01.2004
 Cross bar fwd / rev power meter 27.01.2004 

 
*/

//****************
//include header files

#include	<pic.h>
#include	"swrm.h"


//	__CONFIG(UNPROTECT|BODEN|FOSC1|BKBUG|WRT|CPD);		// CPU config bits v7.xx
	__CONFIG(UNPROTECT&WDTDIS&BOREN&HS&DEBUGDIS&PWRTEN&DUNPROT&PWRTEN&LVPDIS); // config bits for v8.01

/* Global variables */


const char   banner[] = {"    OH2NLT"};		// banner
const char _version[] = {" SWR 27.01.04"};	// system version info

int av;
bit iflag;
bit diagmode;						// send & receive diagnostic messages
unsigned char diag_req;					// diagnostic request op code

#define FWD	0				// FWD power voltage
#define REV	1				// REV power channell
#define REF	3				// 2,5V reference
#define DAC	2				// DAC output voltage

unsigned int fwd;				// forward voltage
unsigned int rev;				// reverse voltage

unsigned int fwd_max;				// forward voltage peak
unsigned int rev_max;				// reverse voltage peak

#define	PSCALE	348				// scaling factor for power display

#define SWR_ALARM 300				// swr alarm lamp limit 300 = 3,00
#define SWR_HIGH 200				// high swr lamp limit

unsigned int p100;				// rev/fwd ratio * 100

unsigned int fwdpwr;				// forward power
unsigned int revpwr;				// reverse power
unsigned int power;				// total power W
unsigned int swr;				// swr * 100

unsigned int peak_fwdpwr;			// peak forward power
unsigned int peak_revpwr;			// peak reverse power
unsigned int peak_power;			// peak total power W
unsigned int peak_swr;				// peak swr * 100


int max_timer;					// max display timer
#define MAX_TIME 2000				// max display delay mS

#define DEF_MODE 1				// default display mode
#define DEF_INT 7				// default display intensity

char d_mode;					// display mode

char dac;					// Voltage reference module (DAC) value



#define MEAN_VAL				// mean value voltage measurement if defined 

/* Include files */

#include	"timers.c"
#include	"lcd8.c"
#include	"adc.c"

// Measure swr voltages & calculate swr

void meas_swr( void )
	{
	int x;
	int f[4];				// mean value calc table
	int r[4];
	long ptemp, ltemp;

#ifdef MEAN_VAL
	f[0] = convert_adc(FWD);		// take four samples
	r[0] = convert_adc(REV);

	f[1] = convert_adc(FWD);
	r[1] = convert_adc(REV);

	f[2] = convert_adc(FWD);
	r[2] = convert_adc(REV);

	f[3] = convert_adc(FWD);
	r[3] = convert_adc(REV);

	fwd = (f[0] + f[1] + f[2] + f[3] + 2) >> 2;	// calculate mean value of
	rev = (r[0] + r[1] + r[2] + r[3]  + 2) >> 2;	// fwd & rev voltages

#else
	fwd = convert_adc(FWD);			// 10.00V = 1000
	rev = convert_adc(REV);
#endif

// calculate current values

	ltemp = fwd;				// calculate forward power
	ptemp = ltemp * ltemp;			// U pwr2
	fwdpwr = ptemp / PSCALE;		// scale for test swr bridge

	ltemp = rev;				// calculate forward power
	ptemp = ltemp * ltemp;			// U pwr2
	revpwr = ptemp / PSCALE;		// scale for test swr bridge

	power = fwdpwr - revpwr;		// calculate outgoing power

	p100 = (rev*100) / fwd;			//  rev/fwd ratio * 100
	x = 100 - p100;
	if(x < 1)
		x = 1;				// prevent divide by zero

	if(fwd >= 10)
		swr = ((100 + p100) * 100) / x;
	else
		swr = 0;			// no swr reading if low power


// calculate peak values

	if((fwd > fwd_max) || (rev > rev_max))	// check if new max
		{
		fwd_max = fwd;
		rev_max = rev;			// set new max values
		max_timer = MAX_TIME;		// set display delay
		}


	ltemp = fwd_max;			// calculate forward power
	ptemp = ltemp * ltemp;			// U pwr2
	peak_fwdpwr = ptemp / PSCALE;		// scale for test swr bridge

	ltemp = rev_max;			// calculate forward power
	ptemp = ltemp * ltemp;			// U pwr2
	peak_revpwr = ptemp / PSCALE;		// scale for test swr bridge

	peak_power = peak_fwdpwr - peak_revpwr;	// calculate outgoing power

	p100 = (rev_max*100) / fwd_max;		//  rev/fwd ratio * 100
	x = 100 - p100;
	if(x < 1)
		x = 1;				// prevent divide by zero

	if(fwd_max >= 10)
		peak_swr = ((100 + p100) * 100) / x;
	else
		peak_swr = 0;			// no swr reading if low power

	}




//***************************
// SWR Meter Main
//***************************

void main( void )
{

char dly;

/* Init IO Ports */

// Set up PORTA
	TRISA = TRISA_INIT;
	PORTA = PORTA_INIT;
	ADCON0 = ADCON0_INIT;
	ADCON1 = ADCON1_INIT;

// Set up PORTB
	TRISB = TRISB_INIT;
	PORTB = PORTB_INIT;

// Set up PORTC
	TRISC = TRISC_INIT;
	PORTC = PORTC_INIT;

// Set up PORTD
	TRISD = TRISD_INIT;
	PORTD = PORTD_INIT;

// Set up PORTE
	TRISE = TRISE_INIT;
	PORTE = PORTE_INIT;

	LEDY = 1;
	LEDR = 1;

#ifdef __A_TYPE
// Set up DAC
	dac = DEF_INT;
	CVRCON = CVRCON_INIT + DEF_INT;
#endif


// init LCD controller

	initlcd();			// init LCD system
	set_chgen();			// set bar graph fonts
	d_mode = DEF_MODE;		// set default display	

// print hello messages

	putst(banner);
	set_cur_lcd(LINE2);
	putst(_version);

	for(dly = 0; dly < 8; dly++)	// show Hello message for a while
		{
		delay( 125 );
		}
	LEDR = 0;
	for(dly = 0; dly < 8; dly++)	// countdown
		{
		delay( 125 );
		}
	LEDY = 0;

	Init_timers();			// init timers & irq system

// start main process loop

for (;;)
	{
	RB2 = 1;		// test
// measure & check alarms

	RB3 = 1;		// test
	meas_swr();					// measure
	RB3 = 0;		// test

	if(peak_swr > SWR_ALARM)			// check swr alarm
		LEDR = 1;
	else
		LEDR = 0;

	if(peak_swr > SWR_HIGH)			// check swr high
		LEDY = 1;
	else
		LEDY = 0;

// do switches

	if(SW1 == 0)
		{
		d_mode++;
		clear_lcd();
		delay(200);				// button debounce / repeat
		}

	if(SW2 == 0)
		{
		clear_lcd();  // test
		}

// do selected display

	switch (d_mode)
		{

		case 1:						// "show everything" display
			set_cur_lcd(LINE1);
			putst("Fwd");
			putintdec(fwdpwr, 1);
			putst(" R");
			putintdec(revpwr, 1);

			set_cur_lcd(LINE2);
			putst("P");
			putintdec(peak_power, 1);

			putst(" SWR");
			if(peak_swr != 0)
				putintdec(peak_swr, 2);
			else
				putst(" -,--");	
			break;

		case 2:						// show power bar graph 0 - 104 W, 1000 / 13 = 76,9
			set_cur_lcd(LINE1);

			if(power>10)
				draw_bar((power/13), 0);	// current situation
			else
				putst("             Pwr");

			set_cur_lcd(LINE2);
			if(peak_power>10)
				draw_bar((peak_power/13), 0);	// peak
			else
				putst("            peak");
			break;

		case 3:						// show fwd/rev bar graph 0 - 104 W, 1000 / 13 = 76,9
			set_cur_lcd(LINE1);

			if(power>10)
				draw_bar((peak_fwdpwr/13), 0); // fwd peak power l to r
			else
				putst("             Fwd");

			set_cur_lcd(LINE2);
			if(peak_power>10)
				draw_bar((peak_revpwr/13), 1);	// rev peak power r to l
			else
				putst("Rev             ");
			break;


		case 4:					// show fwd / rev voltages
			set_cur_lcd(LINE1);
			putst("Fwd V =");
			putintdec(fwd, 2);

			set_cur_lcd(LINE2);
			putst("Rev V =");
			putintdec(rev, 2);
			break;

		case 5:					// Back light DAC or I/O bit
			set_cur_lcd(LINE1);			
			putst("Back Light");
			set_cur_lcd(LINE2);		// print setting
#ifdef __A_TYPE
			putintdec(dac, 0);		// if 877A DAC
#else
			if(BL)
				putst("On  ");		// if 877 on/off control
			else
				putst("Off ");
#endif

			if(SW2 == 0)			// adjust back light with SW2
				{
#ifdef __A_TYPE
				dac++;			// addlust DAC if we have 877A chip
				if(dac > 16)
					{
					dac = 0;
					CVREN = 0;	// ref unit off
					}
				if(dac != 0)
					{
					CVRCON = CVRCON_INIT | (dac - 1);	// set new value
					CVREN = 1;	// ref unit on if it was off
					}
#else
				BL = !BL;		// else on / off toggle
#endif
			set_cur_lcd(LINE2);		// reprint after setting
#ifdef __A_TYPE
			putintdec(dac, 0);		// if 877A DAC
#else
			if(BL)
				putst("On  ");		// if 877 on/off control
			else
				putst("Off ");
#endif
				delay(250);		// button debounce / repeat
				}

			break;


		default: case 0:
			d_mode = 1;			// push to available displays range
			break;
		}

	RB2 = 0;		// test
	}
}

